package com.autentia.tutoriales.cassandra;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.io.UnsupportedEncodingException;
import org.apache.thrift.transport.TTransport;
import org.apache.thrift.transport.TSocket;
import org.apache.thrift.protocol.TProtocol;
import org.apache.thrift.protocol.TBinaryProtocol;
import org.apache.thrift.TException;
import org.apache.cassandra.thrift.Cassandra;
import org.apache.cassandra.thrift.Column;
import org.apache.cassandra.thrift.ColumnOrSuperColumn;
import org.apache.cassandra.thrift.ColumnParent;
import org.apache.cassandra.thrift.ColumnPath;
import org.apache.cassandra.thrift.ConsistencyLevel;
import org.apache.cassandra.thrift.InvalidRequestException;
import org.apache.cassandra.thrift.Mutation;
import org.apache.cassandra.thrift.NotFoundException;
import org.apache.cassandra.thrift.SlicePredicate;
import org.apache.cassandra.thrift.SliceRange;
import org.apache.cassandra.thrift.SuperColumn;
import org.apache.cassandra.thrift.TimedOutException;
import org.apache.cassandra.thrift.UnavailableException;
import org.safehaus.uuid.UUID;
import org.safehaus.uuid.UUIDGenerator;

public class ThriftClient {

	public static final String UTF8 = "UTF8";

	public static final String KEYSPACE = "AutentiaModel";
	public static final String AUTORES_FAMILY = "Autores";
	public static final String BLOGS_FAMILY = "Blogs";
	public static final String ENTRIES_FAMILY = "Entradas";

	public static void main(String[] args) throws TException,
			InvalidRequestException, UnavailableException,
			UnsupportedEncodingException, NotFoundException, TimedOutException {
		
							// Poned aqu vuestra IP.
		TTransport tr = new TSocket("192.168.168.7", 9160);
		TProtocol proto = new TBinaryProtocol(tr);
		Cassandra.Client client = new Cassandra.Client(proto);
		tr.open();
		

		inserting(client);
		reading(client);
		
		
		tr.close();
	}

	private static void inserting(Cassandra.Client client)
			throws UnsupportedEncodingException, InvalidRequestException,
			UnavailableException, TimedOutException, TException {

		
		// Un pseudnimo		
		String keyAutorID = "torero";
		// INSERTAMOS DATOS.
		long timestamp = System.currentTimeMillis();

		// UN AUTOR
		ColumnPath colPathName = new ColumnPath(AUTORES_FAMILY);
		colPathName.setColumn("nombre_completo".getBytes(UTF8));
		client.insert(KEYSPACE, keyAutorID, colPathName,
				"Francisco Javier Martnez Pez".getBytes(UTF8), timestamp,
				ConsistencyLevel.ONE);
		ColumnPath colPathEmail = new ColumnPath(AUTORES_FAMILY);
		colPathEmail.setColumn("email".getBytes(UTF8));

		client.insert(KEYSPACE, keyAutorID, colPathEmail,
				"fjmpaez@autentia.com".getBytes(UTF8), timestamp,
				ConsistencyLevel.ONE);
		
		
		// UN BLOG
		String keyBlogID = "Me gusta Cassandra";
		ColumnPath colBlogName = new ColumnPath(BLOGS_FAMILY);
		colBlogName.setColumn("autor".getBytes(UTF8));
		client.insert(KEYSPACE, keyBlogID, colBlogName,
				keyAutorID.getBytes(UTF8), timestamp,
				ConsistencyLevel.ONE);
				
		// Entradas al Blog		
		List<Column> blogEntriesColumns = new ArrayList<Column>();

		blogEntriesColumns.add(new Column("comentario".getBytes("UTF8"),
		  "Me gusta el futbol, pero tambin me gusta cassandra".getBytes("UTF-8"), timestamp));
		
		blogEntriesColumns.add(new Column("autor".getBytes("UTF8"),
		  "amigo_del_torero".getBytes("UTF-8"), timestamp));
		
		UUID uuid = UUIDGenerator.getInstance().generateTimeBasedUUID();
		
		SuperColumn superColumn = new
		  SuperColumn(uuid.toByteArray(), blogEntriesColumns);
		
		ColumnOrSuperColumn cosc = new ColumnOrSuperColumn();
		cosc.setSuper_column(superColumn);
		
		
		List<Mutation> mutList = new ArrayList<Mutation>();
		Mutation mut = new Mutation();
		mut.setColumn_or_supercolumn(cosc);
		mutList.add(mut);
		
		Map<String,List<Mutation>> mapCF = new HashMap<String,List<Mutation>>(1);
		mapCF.put(ENTRIES_FAMILY, mutList);

		
		Map<String, Map<String, List<Mutation>>> muts = 
		  new HashMap<String, Map<String, List<Mutation>>>();
		muts.put(keyBlogID, mapCF);
		
		client.batch_mutate(KEYSPACE, muts, ConsistencyLevel.ONE);
		
	}

	private static void reading(Cassandra.Client client)
			throws InvalidRequestException, NotFoundException,
			UnavailableException, TimedOutException, TException,
			UnsupportedEncodingException {
		String keyAutorID = "torero";
		String keyBlogID = "Me gusta Cassandra";
		
		
		ColumnPath colPathName = new ColumnPath(AUTORES_FAMILY);
		colPathName.setColumn("nombre_completo".getBytes(UTF8));
		// Leemos una columna (clave, valor y timestamp)
		Column col = client.get(KEYSPACE, keyAutorID, colPathName,
				ConsistencyLevel.ONE).getColumn();

		System.out.println("--------- Leemos una columna ------");
		System.out.println(new String(col.name, UTF8) + ":"
				+ new String(col.value, UTF8) + ":" + new Date(col.timestamp));

		// Leemos las filas completas de autores...
		SlicePredicate predicate = new SlicePredicate();
		SliceRange sliceRange = new SliceRange();
		sliceRange.setStart(new byte[0]);
		sliceRange.setFinish(new byte[0]);
		predicate.setSlice_range(sliceRange);

		System.out.println("--------- Leemos todas las filas de autores ------");
		ColumnParent parent = new ColumnParent(AUTORES_FAMILY);
		List<ColumnOrSuperColumn> results = client.get_slice(KEYSPACE,
				keyAutorID, parent, predicate, ConsistencyLevel.ONE);
		for (ColumnOrSuperColumn result : results) {
			Column column = result.column;
			System.out.print(new String(column.name, UTF8) + ":"
					+ new String(column.value, UTF8) + ":"
					+ new Date(column.timestamp)+" | ");
		}
		System.out.println("----------------------------");
		// Leemos las entradas del blog 
		System.out.println("--------- Leemos todas los comentarios del blog ------");
		parent = new ColumnParent(ENTRIES_FAMILY);
		
		results = client.get_slice(KEYSPACE, keyBlogID, parent, predicate, ConsistencyLevel.ONE);

		    for (ColumnOrSuperColumn result : results) {
		      SuperColumn superColumn = result.super_column;
		      UUID uuid = new UUID(superColumn.name);
		      System.out.println(uuid.toString() + " [ ");
		      for (Column column : superColumn.columns) {
		    	  System.out.print(new String(column.name, UTF8) + ":"
							+ new String(column.value, UTF8) + ":"
							+ new Date(column.timestamp)+" | ");
		      }
		      System.out.println(" ] ");
		    }
		
		
	}

}